#include <Servo.h>
#include <SharpIR.h>

#define IR_SENSOR A0 // Sharp IR GP2Y0A21YK0F (10-80cm, analog)
#define SERVOMOTOR_PWM 3
#define SERVO_LEFT 10
#define SERVO_RIGHT 170
#define SERVO_FORWARD 115

#define MOTOR_RIGHT_FORWARD 4
#define MOTOR_RIGHT_REVERSE 12
#define MOTOR_RIGHT_PWM 5

#define MOTOR_LEFT_FORWARD 8
#define MOTOR_LEFT_REVERSE 7
#define MOTOR_LEFT_PWM 6

#define LED_BLUE 9
#define LED_RED 10
#define LED_GREEN 11

#define BUTTON 2

Servo servo;
int motorSpeed = 175;
int motorOffset = 35;
int turnSpeed = 175;
int stopDistance = 45;
int lateralDistance = 30;
int speedCnt;
int speedStep = 10;
int goesForward = 0;

int spotDistance() {
  float voltage = analogRead(IR_SENSOR) * (5.0 / 1023.0);
  float distance = 29.988 * pow(voltage, -1.173);

  return (int)distance;
}

int getDistance() {
  int sum = 0, count = 25;
  for (int i = 0; i < 25; ++i) {
    sum += spotDistance();
  }

  return sum / count;
}

int checkForward() {
  delay(500);
  int distance = getDistance();
  delay(200);
  return distance;
}

int checkLeft() {
  servo.write(SERVO_LEFT);
  delay(1000);
  int distance = getDistance();
  delay(500);
  servo.write(SERVO_FORWARD);
  delay(500);
  return distance;
}

int checkRight() {
  servo.write(SERVO_RIGHT);
  delay(1000);
  int distance = getDistance();
  delay(500);
  servo.write(SERVO_FORWARD);
  delay(500);
  return distance;
}

void moveForward() {
  analogWrite(MOTOR_RIGHT_PWM, motorSpeed);
  analogWrite(MOTOR_LEFT_PWM, motorSpeed + motorOffset);
  
  digitalWrite(MOTOR_RIGHT_FORWARD, 1);
  digitalWrite(MOTOR_RIGHT_REVERSE, 0);
  digitalWrite(MOTOR_LEFT_FORWARD, 1);
  digitalWrite(MOTOR_LEFT_REVERSE, 0);
}

void turnLeft(int duration) {
  goesForward = 0;
  analogWrite(MOTOR_RIGHT_PWM, turnSpeed);
  analogWrite(MOTOR_LEFT_PWM, turnSpeed);

  digitalWrite(MOTOR_RIGHT_FORWARD, 1);
  digitalWrite(MOTOR_RIGHT_REVERSE, 0);
  digitalWrite(MOTOR_LEFT_FORWARD, 0);
  digitalWrite(MOTOR_LEFT_REVERSE, 1);

  delay(duration);
}

void turnRight(int duration) {
  goesForward = 0;
  analogWrite(MOTOR_RIGHT_PWM, turnSpeed);
  analogWrite(MOTOR_LEFT_PWM, turnSpeed);

  digitalWrite(MOTOR_RIGHT_FORWARD, 0);
  digitalWrite(MOTOR_RIGHT_REVERSE, 1);
  digitalWrite(MOTOR_LEFT_FORWARD, 1);
  digitalWrite(MOTOR_LEFT_REVERSE, 0);

  delay(duration);
}

void moveStop() {
  goesForward = 0;
  digitalWrite(MOTOR_RIGHT_FORWARD, 0);
  digitalWrite(MOTOR_RIGHT_REVERSE, 0);
  digitalWrite(MOTOR_LEFT_FORWARD, 0);
  digitalWrite(MOTOR_LEFT_REVERSE, 0);
}

int start = 0;
void power() {
  Serial.println("hello");
  start = 1;
}

void setRGB(int red, int green, int blue) {
  analogWrite(LED_RED, red);
  analogWrite(LED_GREEN, green);
  analogWrite(LED_BLUE, blue);
}

void setup() {
  Serial.begin(9600);
  servo.attach(SERVOMOTOR_PWM);

  pinMode(MOTOR_RIGHT_FORWARD, OUTPUT);
  pinMode(MOTOR_RIGHT_REVERSE, OUTPUT);
  pinMode(MOTOR_RIGHT_PWM, OUTPUT);
  pinMode(MOTOR_LEFT_FORWARD, OUTPUT);
  pinMode(MOTOR_LEFT_REVERSE, OUTPUT);
  pinMode(MOTOR_LEFT_PWM, OUTPUT);

  pinMode(LED_RED, OUTPUT);
  pinMode(LED_GREEN, OUTPUT);
  pinMode(LED_BLUE, OUTPUT);

  pinMode(BUTTON, INPUT_PULLUP);
  digitalWrite(LED_BLUE, 1);
  digitalWrite(LED_GREEN, 1);
  digitalWrite(LED_RED, 1);
  attachInterrupt(digitalPinToInterrupt(BUTTON), power, FALLING);

  while (start == 0) {
    delay(500);
  }
}

int distanceForward, distanceLeft, distanceRight;

void loop() {
  setRGB(0, 255, 0);

  distanceForward = checkForward();
  if (distanceForward <= stopDistance) {
    setRGB(255, 0, 0);
    moveStop();
    distanceLeft = checkLeft();
    delay(500);
    distanceRight = checkRight();
    delay(500);

    if (distanceLeft <= lateralDistance && distanceRight <= lateralDistance) {
      // All ways are blocked, turn around
      setRGB(0, 0, 255);
      turnLeft(750);
    } else {
      if (distanceLeft > distanceRight) {
        setRGB(0, 0, 255);
        turnRight(500);
        setRGB(255, 0, 0);
        moveStop();
      } else {
        setRGB(0, 0, 255);
        turnLeft(500);
        setRGB(255, 0, 0);
        moveStop();
      }
    }
  } else {
    setRGB(0, 255, 0);
    moveForward();
  }
}
